/* global VuFind */
'use strict'

// NOTE: Required packages/scripts:
// css-element-queries/src/ResizeSensor
// slim-select
// sticky-sidebar

const theme = {
  scrollPos: 0,
  sidebar: null,
  viewportWidth: window.innerWidth,
  init() {
    theme.initSelects()

    // Sticky sidebar
    // NOTE: Requires ResizeSensor for automatic resizing
    const sidebar = document.querySelector('#sidebar')
    if (sidebar) {
      // eslint-disable-next-line no-new
      new StickySidebar('#sidebar', {
        topSpacing: 28,
        bottomSpacing: 0,
        containerSelector: '#sidebar-container',
        innerWrapperSelector: '#sidebar-inner',
        minWidth: 1024, // NOTE: Must match the `$bp4` breakpoint (see scss/util/settings.scss)
      })
    }

    theme.updateScrolled()
    window.addEventListener('scroll', theme.updateScrolled)

    // Close open sidebar when viewport size changes
    window.addEventListener('resize', () => {
      if (theme.viewportWidth === window.innerWidth) {
        return
      }

      theme.viewportWidth = window.innerWidth

      if (theme.sidebar && theme.sidebar.classList.contains('-open')) {
        theme.toggleSidebar()
      }
    })

    // Handle aria-based toggles
    window.addEventListener('click', (event) => {
      const button = event.target.closest('a, button')
      if (!button) {
        return
      }

      const targetId = button.getAttribute('aria-controls')
      if (!targetId) {
        return
      }

      const role = button.getAttribute('role')
      const ariaExpanded = button.getAttribute('aria-expanded')

      if (role === 'tab') {
        // NOTE: Handled by Bootstrap JS
      } else if (ariaExpanded !== null) {
        button.setAttribute('aria-expanded', ariaExpanded === 'false')
        const targetElement = document.getElementById(targetId)
        if (ariaExpanded === 'false') {
          targetElement.removeAttribute('hidden')
        } else {
          targetElement.setAttribute('hidden', '')
        }
      } else {
        theme.toggleSidebar(targetId)
      }
    })
  },
  initSelects(parentNode = document) {
    const selects = parentNode.querySelectorAll('select')

    for (let i = 0; i < selects.length; i++) {
      const select = selects[i]
      const isMultiSelect = select.hasAttribute('multiple')

      if (!isMultiSelect && select.childElementCount < 6) {
        continue
      }

      // Exclude "type" select in advanced search; these fields are cloned,
      // so SlimSelect would have to be re-initialized
      if (select.classList.contains('adv-term-type')) {
        continue
      }

      new SlimSelect({ // eslint-disable-line no-new
        select,
        closeOnSelect: !isMultiSelect,
        placeholder: VuFind.translate('slimselect_placeholder'),
        searchText: VuFind.translate('slimselect_empty_result'),
        searchPlaceholder: 'Filter',
        searchFocus: true,
        searchHighlight: true,
        deselectLabel: '&times;',
        hideSelectedOption: isMultiSelect,
      })
    }
  },
  scrollToTop() {
    const duration = .25
    const element = document.documentElement.scrollTop ? document.documentElement : document.body
    const scrollTop = element.scrollTop

    if (scrollTop < 0) {
      return false
    }

    let currentTime = 0

    const animationFrame = (() => {
      return window.requestAnimationFrame ||
        window.webkitRequestAnimationFrame ||
        window.mozRequestAnimationFrame ||
        function (callback) { window.setTimeout(callback, 1000 / 60) }
    })()

    function easeInOutSine(pos) {
      return (-0.5 * (Math.cos(Math.PI * pos) - 1))
    }

    // Animation loop
    function tick() {
      currentTime += 1 / 60
      const p = currentTime / duration
      const t = easeInOutSine(p)

      // NOTE: scrollTo() does not work in IE
      if (p < 1) {
        animationFrame(tick)
        element.scrollTop = scrollTop + ((0 - scrollTop) * t)
      } else {
        element.scrollTop = 0
      }
    }

    tick()

    return false
  },
  toggleFacets() {
    event.target.parentElement.classList.toggle('-collapsed')
  },
  toggleSidebar(id) {
    if (id) {
      theme.sidebar = document.getElementById(id)
    }

    if (!theme.sidebar) {
      return
    }

    if (!theme.sidebar.classList.contains('-open')) {
      // Focus input when sidebar is halfway visible
      const input = theme.sidebar.querySelector('[data-focus-off-canvas]')
      if (input) {
        setTimeout(() => { input.select() }, 125)
      }

      theme.scroll.lock()
    } else {
      theme.scroll.unlock()
    }

    theme.sidebar.classList.toggle('-open')

    // TODO: aria-expanded should be set, but multiple buttons may be affected
  },
  // Toggling "scrolled" for to-top button
  updateScrolled() {
    if (document.documentElement.scrollTop > 84 || document.body.scrollTop > 84) {
      document.body.classList.add('-scrolled')
    } else {
      document.body.classList.remove('-scrolled')
    }
  },
  scroll: {
    lock() {
      theme.scrollPos = document.documentElement.scrollTop || document.body.scrollTop
      document.body.style.top = `-${theme.scrollPos}px`
      document.body.style.marginBottom = `-${theme.scrollPos}px`
      document.body.style.height = `calc(100vh + ${theme.scrollPos}px)`
      document.body.style.overflow = 'hidden'
      document.body.style.position = 'relative'
    },
    unlock() {
      document.body.style = ''

      setTimeout(() => {
        document.documentElement.scrollTop = document.body.scrollTop = theme.scrollPos
      })
    },
  },
}

window.addEventListener('load', theme.init)

// Make `theme` globally available
window.theme = theme
